/*
 *  Copyright (c) 2007 André Queimado, João Abecasis
 *
 *  Data:       Fri 9 March 2007
 *
 *  Autor:      André Queimado <aaiq@caixamagica.pt>,
 *              João Abecasis <jpra@caixamagica.pt>
 *
 *              Caixa Mágica Software
 *
 *  This file is part of CM-INSTALL.
 *
 *  CM-INSTALL is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License a published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  CM-INSTALL is distributed in the hope that it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of #MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 *  more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with Foobar; if not, write to the Free Software #Foundation, Inc., 51
 *  Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 
 */

#include <glib-object.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <regex.h>
#include <gdk/gdk.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>

#define MAX_FILENAME 30

GtkWidget *mainwindow;
GtkWidget *intro_window;
GtkWidget *assistente_instalacao;
GtkWidget *progresso_pacote;
GtkWidget *progresso_geral;
GtkWidget *botao_fechar;
/*GtkWidget *botao_cancelar_apt;*/
GtkWidget *botao_cancelar;
GtkWidget *botao_confirmar;
GtkWidget *view_status_instalacao;
GtkWidget *view_repositorios;
GtkWidget *label_pacote_a_instalar;
GtkWidget *label_repositorios;
GtkWidget *label_mensagem;
GtkWidget *expander_status;
GtkWidget *button_box;
GtkTextBuffer *buffer_status_instalacao;
GtkTextBuffer *buffer_repositorios;

GtkAdjustment *ajustamento_vertical_scroll;

int contador_pacote;
int contador_geral;
int numero_pacotes;

int pacotes_so_far;
int pacotes_adicionais;

int percentagem_progresso;
int state=0;

gdouble avanco_pacote;
gdouble avanco_geral;

gdouble barra_pacote;
gdouble barra_geral;

GPid pid_aplicacao = 0;

gboolean info_pacotes_obtida = FALSE;
gboolean pacotes_em_instalacao = FALSE;
gboolean sucesso_instalacao = TRUE;
gboolean pacote_ja_instalado = FALSE;
gboolean time_out_parado = FALSE;

GString *legenda_pacote = NULL;
GString *buffer_numero_pacotes = NULL;
GString *legenda_pacote_2 = NULL;

gchar *pacote = NULL;

gchar * pacote_a_instalar = NULL;

enum estados_parsing
{
    ESTADO_PROCURA_UPGRADED,
    ESTADO_PROCURA_COMMITTING,
    ESTADO_CONTA_CARDINAIS,
    ESTADO_REPORT
};

void verifica_estado_aplicacao(GPid pid, gint status, gpointer data)
{
    if (WIFEXITED(status) && !WEXITSTATUS(status))
        sucesso_instalacao = TRUE;
    else
        sucesso_instalacao = FALSE;

    g_spawn_close_pid(pid);
}

void personaliza_botao(GtkWidget * widget, gpointer data)
{
    GdkColor cor_botao;

    gdk_color_parse("white", &cor_botao);

    GtkWidget *imagem_botao_dialog_ok =
        gtk_image_new_from_file("/usr/share/cm-install/pixmaps/ok.png");

    gtk_button_set_image(GTK_BUTTON(widget), imagem_botao_dialog_ok);
    gtk_button_set_relief(GTK_BUTTON(widget), GTK_RELIEF_NONE);

    gtk_widget_modify_bg(widget, GTK_STATE_PRELIGHT, &cor_botao);
    gtk_button_set_focus_on_click(GTK_BUTTON(widget), TRUE);
}

void enviaMensagemErro(gchar * erro)
{
    GtkWidget *dialog_erro =
        gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
        GTK_BUTTONS_NONE, "Erro:\n%s", erro);

    gtk_dialog_add_buttons(GTK_DIALOG(dialog_erro), "", GTK_RESPONSE_NONE,
        NULL);

    GdkColor cor_janela;

    gdk_color_parse("white", &cor_janela);

    gtk_widget_modify_bg(dialog_erro, GTK_STATE_NORMAL, &cor_janela);

    gtk_container_forall(GTK_CONTAINER(GTK_DIALOG(dialog_erro)->action_area),
        personaliza_botao, NULL);

    gtk_window_set_position (GTK_WINDOW(dialog_erro),GTK_WIN_POS_CENTER);

    gtk_dialog_run(GTK_DIALOG(dialog_erro));
    gtk_widget_destroy(dialog_erro);
}

void enviaMensagemAviso(gchar * aviso)
{
    GtkWidget *dialog_aviso =
        gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
        GTK_BUTTONS_NONE, "Aviso:\n%s", aviso);

    gtk_dialog_add_buttons(GTK_DIALOG(dialog_aviso), "", GTK_RESPONSE_NONE,
        NULL);

    GdkColor cor_janela;

    gdk_color_parse("white", &cor_janela);

    gtk_widget_modify_bg(dialog_aviso, GTK_STATE_NORMAL, &cor_janela);

    gtk_container_forall(GTK_CONTAINER(GTK_DIALOG(dialog_aviso)->action_area),
        personaliza_botao, NULL);

    gtk_window_set_position (GTK_WINDOW(dialog_aviso),GTK_WIN_POS_CENTER);

    gtk_dialog_run(GTK_DIALOG(dialog_aviso));

    gdouble valor_scroll_;
    gdouble valor_janela_;
    gdouble valor_novo_;

    if (!gtk_expander_get_expanded(GTK_EXPANDER(expander_status)))
        gtk_expander_set_expanded(GTK_EXPANDER(expander_status), TRUE);

    g_object_get(ajustamento_vertical_scroll, "upper", &valor_scroll_, NULL);
    g_object_get(ajustamento_vertical_scroll, "page-size", &valor_janela_,
        NULL);

    valor_novo_ = valor_scroll_ - valor_janela_;

    g_object_set(ajustamento_vertical_scroll, "value", valor_novo_, NULL);

    gtk_adjustment_value_changed(GTK_ADJUSTMENT(ajustamento_vertical_scroll));

    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progresso_geral), "");

    gtk_widget_queue_draw(mainwindow);

    gtk_widget_set_sensitive(GTK_WIDGET(botao_fechar), TRUE);

    gtk_widget_destroy(dialog_aviso);
}

gboolean pulsa(gpointer i)
{
    if (time_out_parado)
    {
        gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progresso_pacote), 0.);
        return FALSE;
    }
    else
    {
        gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progresso_pacote));
        return TRUE;
    }
}

/*void cancelaApt(GtkButton * button, gpointer data)*/
/*{*/
/*    kill (pid_aplicacao, SIGTERM);*/
/*}*/

gboolean indicaProgresso(gpointer i)
{
    static GIOChannel *canal_output = NULL;
    static enum estados_parsing estado_actual = ESTADO_PROCURA_UPGRADED;

    if (canal_output == NULL)
    {
        contador_pacote = 0;
        contador_geral = 0;
        avanco_pacote = 1. / 50.;
        numero_pacotes = 0;
        pacotes_so_far = 0;
        avanco_geral = 0;
        barra_pacote = 0;
        barra_geral = 0;
        int script_stdout = 0;

        

        gchar *script_argv[] =
            { "/usr/share/cm-install/cm-install-script",
            pacote_a_instalar, NULL
        };

        g_spawn_async_with_pipes(NULL, script_argv, NULL,
            G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid_aplicacao, NULL,
            &script_stdout, NULL, NULL);

        g_child_watch_add(pid_aplicacao, verifica_estado_aplicacao, NULL);

        fcntl(script_stdout, F_SETFL, O_NONBLOCK);

        canal_output = g_io_channel_unix_new(script_stdout);
        legenda_pacote = g_string_new("");
        legenda_pacote_2 = g_string_new("");
    }

    gchar caracter_controlo[2] = "J";
    gchar caracter_controlo_2[2] = "J";
    gsize bytes_lidos = 0;
    gchar *linha_controlo = NULL;
    gchar *linha_controlo_2 = NULL;

    GIOStatus status = G_IO_STATUS_ERROR;

    switch (estado_actual)
    {
    case ESTADO_PROCURA_UPGRADED:
        status =
            g_io_channel_read_chars(canal_output, caracter_controlo_2, 1,
            &bytes_lidos, NULL);
        if (status == G_IO_STATUS_NORMAL)
        {
            legenda_pacote_2 =
                g_string_append(legenda_pacote_2, caracter_controlo_2);

            if (*caracter_controlo_2 == '\n')
            {
                gchar *conteudo_antigo_2;

                linha_controlo_2 = g_strdup(legenda_pacote_2->str);

                legenda_pacote_2 = g_string_erase(legenda_pacote_2, 0, -1);

                g_object_get(buffer_status_instalacao, "text",
                    &conteudo_antigo_2, NULL);

                gchar *string_nova_2 =
                    g_strconcat(conteudo_antigo_2, linha_controlo_2, NULL);

                gtk_text_buffer_set_text(buffer_status_instalacao,
                    string_nova_2, -1);

                gdouble valor_scroll;
                gdouble valor_janela;
                gdouble valor_novo;

                g_object_get(ajustamento_vertical_scroll, "upper",
                    &valor_scroll, NULL);
                g_object_get(ajustamento_vertical_scroll, "page-size",
                    &valor_janela, NULL);

                valor_novo = valor_scroll - valor_janela;

                g_object_set(ajustamento_vertical_scroll, "value", valor_novo,
                    NULL);

                gtk_adjustment_value_changed(GTK_ADJUSTMENT
                    (ajustamento_vertical_scroll));

                g_free(string_nova_2);
                g_free(conteudo_antigo_2);

                gchar *regex_upgraded =
                    "\\([[:digit:]]\\+\\) upgraded, \\([[:digit:]]\\+\\) newly installed, [[:digit:]]\\+ to remove and [[:digit:]]\\+ not upgraded."; 

                regex_t re;

                regmatch_t match[20];

                regcomp(&re, regex_upgraded, REG_NEWLINE); 

                if (!regexec(&re, linha_controlo_2,
                        sizeof(match) / sizeof(match[0]), match, 0))
                {
                    pacotes_so_far =
                        g_ascii_strtoll(linha_controlo_2 + match[1].rm_so, NULL,
                        10) + g_ascii_strtoll(linha_controlo_2 + match[2].rm_so,
                        NULL, 10);

                    numero_pacotes = numero_pacotes + pacotes_so_far;

                    if (numero_pacotes == 0)
                    {
                        pacote_ja_instalado = TRUE;
                        estado_actual = ESTADO_REPORT;
                    }

                    numero_pacotes = numero_pacotes + 1;
                    avanco_geral = avanco_pacote / numero_pacotes * .9;

                    estado_actual = ESTADO_PROCURA_COMMITTING;
                }
            }
        }

        if (!sucesso_instalacao)
            estado_actual = ESTADO_REPORT;

        g_free(linha_controlo_2);
        break;

    case ESTADO_PROCURA_COMMITTING:
        status =
            g_io_channel_read_line(canal_output, &linha_controlo, NULL, NULL,
            NULL);

        if (status == G_IO_STATUS_NORMAL)
            if (g_strrstr(linha_controlo, "Committing changes..."))
                estado_actual = ESTADO_CONTA_CARDINAIS;

        if (!sucesso_instalacao)
            estado_actual = ESTADO_REPORT;

        break;

    case ESTADO_CONTA_CARDINAIS:
        status =
            g_io_channel_read_chars(canal_output, caracter_controlo, 1,
            &bytes_lidos, NULL);

        if (status == G_IO_STATUS_NORMAL)
        {
            time_out_parado = TRUE;

            if (*caracter_controlo != ' ' && *caracter_controlo != '#'
                && *caracter_controlo != '\n')
                legenda_pacote =
                    g_string_append(legenda_pacote, caracter_controlo);

            if (*caracter_controlo == '#')
            {
                gchar legenda_pacote_instalado[100] = "";

                g_snprintf(legenda_pacote_instalado, 100, "A instalar %s ",
                    legenda_pacote->str);

		/* if (g_strrstr(legenda_pacote_instalado, "Preparing...")) */
                if (g_strrstr(legenda_pacote_instalado, "Preparing"))
                    gtk_label_set_label(GTK_LABEL(label_mensagem),
                        "A preparar...");

                else
                    gtk_label_set_label(GTK_LABEL(label_mensagem),
                        legenda_pacote_instalado);

                barra_pacote = barra_pacote + avanco_pacote;

                barra_geral = barra_geral + avanco_geral;

                if (barra_pacote > 1.0)
                {
                    barra_pacote = 1.0;
                    legenda_pacote = g_string_erase(legenda_pacote, 0, -1);
                }

                gdouble progresso_d = barra_geral * 100;

                percentagem_progresso = (int)progresso_d;
                gchar legenda_percentagem[20] = "";

                g_snprintf(legenda_percentagem, 20, "Concluído %i %%",
                    percentagem_progresso);

                gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progresso_geral),
                    legenda_percentagem);

                gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR
                    (progresso_pacote), barra_pacote);
                gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progresso_geral),
                    barra_geral);

                if (percentagem_progresso >= 90)
                    estado_actual = ESTADO_REPORT;

                if (!sucesso_instalacao)
                    estado_actual = ESTADO_REPORT;
            }
            else
                barra_pacote = 0;
        }
        break;

    case ESTADO_REPORT:
        status =
            g_io_channel_read_line(canal_output, &linha_controlo, NULL, NULL,
            NULL);

        gchar *conteudo_antigo_2;

        g_object_get(buffer_status_instalacao, "text", &conteudo_antigo_2,
            NULL);

        gchar *string_nova_2 =
            g_strconcat(conteudo_antigo_2, linha_controlo, NULL);

        gtk_text_buffer_set_text(buffer_status_instalacao, string_nova_2, -1);

        gdouble valor_scroll_2;
        gdouble valor_janela_2;
        gdouble valor_novo_2;

        g_object_get(ajustamento_vertical_scroll, "upper", &valor_scroll_2,
            NULL);
        g_object_get(ajustamento_vertical_scroll, "page-size", &valor_janela_2,
            NULL);

        valor_novo_2 = valor_scroll_2 - valor_janela_2;

        g_object_set(ajustamento_vertical_scroll, "value", valor_novo_2, NULL);

        gtk_adjustment_value_changed(GTK_ADJUSTMENT
            (ajustamento_vertical_scroll));

        g_free(string_nova_2);
        g_free(conteudo_antigo_2);

        break;
    }

    g_free(linha_controlo);

    if (status == G_IO_STATUS_NORMAL || status == G_IO_STATUS_AGAIN)
        return TRUE;
    else
    {
        g_io_channel_close(canal_output);
        g_string_free(legenda_pacote, TRUE);

        if (!time_out_parado)
            time_out_parado = TRUE;

        if (sucesso_instalacao == FALSE)
        {
            gtk_label_set_label(GTK_LABEL(label_mensagem),
                "Ocorreu um erro durante a instalação");
            enviaMensagemAviso
                ("Ocorreu um erro durante a instalação.\nA instalação foi abortada");
	    state=1;
        }
        else
        {
            if (!pacote_ja_instalado)
            {
                gtk_label_set_label(GTK_LABEL(label_mensagem),
                    "Instalação terminada com sucesso");
                gtk_progress_bar_set_text(GTK_PROGRESS_BAR(progresso_geral),
                    "Concluído 100%");

                gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progresso_geral),
                    1.0);

                gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR
                    (progresso_pacote), 1.0);
            }
            else
            {
                gchar *string_da_label_2 =
                    g_strconcat("O pacote ", pacote, NULL);
                string_da_label_2 =
                    g_strconcat(string_da_label_2,
                    " já se encontra instalado.\nA instalação foi abortada",
                    NULL);

                enviaMensagemAviso(string_da_label_2);
                gtk_label_set_label(GTK_LABEL(label_mensagem),
                    string_da_label_2);
            }
        }

/*        gtk_container_remove (GTK_CONTAINER(button_box), botao_cancelar_apt);*/
/*        gtk_box_pack_start(GTK_BOX(button_box), botao_fechar, FALSE, FALSE, 0);*/
        gtk_widget_set_sensitive(GTK_WIDGET(botao_fechar), TRUE);
        gtk_widget_show(botao_fechar);
        return FALSE;
    }
}

void iniciaProgresso(GtkButton * button, gpointer data)
{
    gtk_widget_hide_all(intro_window);
    gtk_widget_show_all(mainwindow);
    g_idle_add(indicaProgresso, NULL);
    g_timeout_add(250, pulsa, NULL);
}

void retError()
{
	gtk_main_quit();
	exit(1);
}

void retError2()
{
	gtk_main_quit();
	if (state != 0){
	exit(1);}
}

int main(int argc, char *argv[])
{
    gtk_init(&argc, &argv);

   if (argc < 2)
   {
	   enviaMensagemErro("Não foi possível abrir o ficheiro");
	   return 1;
   }

   int aux = argc - 1;
   int ij = 0;
   int pos = 0;
   pacote_a_instalar = "";
   for(ij = 1; ij <= aux; ij++){
	    gchar * text = malloc(sizeof(gchar) * (strlen(argv[ij]) + 1));
	    strncpy(text, argv[ij], strlen(argv[ij])+1);
	    pacote_a_instalar = g_strconcat(pacote_a_instalar, text, NULL);
	    if (aux > 1 && ij != aux){
		    pacote_a_instalar = g_strconcat(pacote_a_instalar, " ", NULL);
	    }
    }

    mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    gtk_window_set_position (GTK_WINDOW(mainwindow),GTK_WIN_POS_CENTER);

    GdkColor color;

    gdk_color_parse("white", &color);

    gtk_widget_modify_bg(mainwindow, GTK_STATE_NORMAL, &color);

    GtkWidget *vbox = gtk_vbox_new(FALSE, 10);

    GtkWidget *vbox_progressos = gtk_vbox_new(FALSE, 6);
    button_box = gtk_hbutton_box_new();

    gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_END);

    GtkWidget *banner_intro;

    banner_intro =
        gtk_image_new_from_file
        ("/usr/share/cm-install/pixmaps/cminstall2.png");
    gtk_window_set_title(GTK_WINDOW(mainwindow), "CM Install");
    gtk_window_set_default_size(GTK_WINDOW(mainwindow), 444, 458);
    gtk_window_set_resizable(GTK_WINDOW(mainwindow), FALSE);

    progresso_pacote = gtk_progress_bar_new();
    progresso_geral = gtk_progress_bar_new();

    expander_status = gtk_expander_new("Detalhes");

    GtkWidget *janela_status = gtk_scrolled_window_new(NULL, NULL);

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(janela_status),
        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_widget_set_usize(GTK_WIDGET(janela_status), -1, 100);

    ajustamento_vertical_scroll =
        gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(janela_status));

    view_status_instalacao = gtk_text_view_new();
    gtk_text_view_set_editable(GTK_TEXT_VIEW(view_status_instalacao), FALSE);
    buffer_status_instalacao =
        gtk_text_view_get_buffer(GTK_TEXT_VIEW(view_status_instalacao));
    gtk_text_buffer_set_text(buffer_status_instalacao, "\n", -1);

    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(janela_status),
        view_status_instalacao);

    gtk_container_add(GTK_CONTAINER(expander_status), janela_status);

    botao_fechar = gtk_button_new();
/*    botao_cancelar_apt = gtk_button_new();*/
    GtkWidget *imagem_botao_fechar;
    GtkWidget *imagem_botao_cancelar_apt;

    gtk_button_box_set_child_size(GTK_BUTTON_BOX(button_box), 60, 21);

    imagem_botao_fechar =
        gtk_image_new_from_file
        ("/usr/share/cm-install/pixmaps/fechar.png");
    gtk_button_set_image(GTK_BUTTON(botao_fechar), imagem_botao_fechar);
    gtk_button_set_relief(GTK_BUTTON(botao_fechar), GTK_RELIEF_NONE);

    imagem_botao_cancelar_apt =
        gtk_image_new_from_file
        ("/usr/share/cm-install/pixmaps/cancel.png");
/*    gtk_button_set_image(GTK_BUTTON(botao_cancelar_apt), imagem_botao_cancelar_apt);*/
/*    gtk_button_set_relief(GTK_BUTTON(botao_cancelar_apt), GTK_RELIEF_NONE);*/

    gtk_widget_modify_bg(botao_fechar, GTK_STATE_PRELIGHT, &color);
    gtk_widget_set_sensitive(GTK_WIDGET(botao_fechar), FALSE);

    gtk_button_set_focus_on_click(GTK_BUTTON(botao_fechar), TRUE);

/*    gtk_widget_modify_bg(botao_cancelar_apt, GTK_STATE_PRELIGHT, &color);*/

    gtk_button_set_focus_on_click(GTK_BUTTON(botao_fechar), TRUE);

    label_mensagem = gtk_label_new("Aguarde um momento");
    gtk_label_set_justify(GTK_LABEL(label_mensagem), GTK_JUSTIFY_LEFT);

    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

    gtk_box_pack_start(GTK_BOX(vbox_progressos), progresso_pacote, FALSE, FALSE,
        0);
    gtk_box_pack_start(GTK_BOX(vbox_progressos), progresso_geral, FALSE, FALSE,
        0);

    

    gtk_box_pack_start(GTK_BOX(button_box), botao_fechar, FALSE, FALSE, 0);

    gtk_container_add(GTK_CONTAINER(mainwindow), vbox);
    gtk_box_pack_start(GTK_BOX(vbox), banner_intro, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), label_mensagem, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), expander_status, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), vbox_progressos, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), button_box, FALSE, FALSE, 0);

    //if (state == 0) {g_signal_connect(botao_fechar, "clicked", gtk_main_quit, NULL);}
    g_signal_connect(botao_fechar, "clicked", G_CALLBACK(retError2), NULL);

    g_signal_connect(mainwindow, "delete-event", gtk_main_quit, NULL);

/*    g_signal_connect(botao_cancelar_apt, "clicked", G_CALLBACK(cancelaApt),*/
/*        NULL);*/

/******************************************************************************
 ******************************************************************************
 ******************************************************************************
 ******************************************************************************/

    intro_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_position (GTK_WINDOW(intro_window),GTK_WIN_POS_CENTER);
    gtk_widget_modify_bg(intro_window, GTK_STATE_NORMAL, &color);
    GtkWidget *vbox_repositorios = gtk_vbox_new(FALSE, 10);
    GtkWidget *button_box_ok_cancel = gtk_hbutton_box_new();

    gtk_container_set_border_width(GTK_CONTAINER(vbox_repositorios), 10);

    GtkWidget *banner_intro2;

    banner_intro2 =
        gtk_image_new_from_file
        ("/usr/share/cm-install/pixmaps/cminstall2.png");
    gtk_window_set_title(GTK_WINDOW(intro_window), "CM Install");
    gtk_window_set_default_size(GTK_WINDOW(intro_window), 444, 458);
    gtk_window_set_resizable(GTK_WINDOW(intro_window), FALSE);

    label_pacote_a_instalar = gtk_label_new("A ler pacote...");
    label_repositorios =
        gtk_label_new
        ("Foram escolhidos os seguintes repositórios para a instalação:");

    gtk_misc_set_alignment(GTK_MISC(label_pacote_a_instalar), 0.f, 0.f);
    gtk_misc_set_alignment(GTK_MISC(label_repositorios), 0.f, 0.f);

    GtkWidget *janela_repositorios = gtk_scrolled_window_new(NULL, NULL);

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(janela_repositorios),
        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    view_repositorios = gtk_text_view_new();
    gtk_text_view_set_editable(GTK_TEXT_VIEW(view_repositorios), FALSE);
    buffer_repositorios =
        gtk_text_view_get_buffer(GTK_TEXT_VIEW(view_repositorios));
    gtk_text_buffer_set_text(buffer_repositorios, "\n", -1);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW
        (janela_repositorios), view_repositorios);

    botao_cancelar = gtk_button_new();
    GtkWidget *imagem_botao_cancelar;

    gtk_button_box_set_child_size(GTK_BUTTON_BOX(button_box_ok_cancel), 60, 21);

    imagem_botao_cancelar =
        gtk_image_new_from_file
        ("/usr/share/cm-install/pixmaps/cancel.png");
    gtk_button_set_image(GTK_BUTTON(botao_cancelar), imagem_botao_cancelar);
    gtk_button_set_relief(GTK_BUTTON(botao_cancelar), GTK_RELIEF_NONE);

    botao_confirmar = gtk_button_new();
    GtkWidget *imagem_botao_confirmar;

    imagem_botao_confirmar =
        gtk_image_new_from_file("/usr/share/cm-install/pixmaps/ok.png");
    gtk_button_set_image(GTK_BUTTON(botao_confirmar), imagem_botao_confirmar);
    gtk_button_set_relief(GTK_BUTTON(botao_confirmar), GTK_RELIEF_NONE);

    gtk_widget_modify_bg(botao_cancelar, GTK_STATE_PRELIGHT, &color);
    gtk_widget_modify_bg(botao_confirmar, GTK_STATE_PRELIGHT, &color);

    gtk_box_pack_start(GTK_BOX(button_box_ok_cancel), botao_cancelar, FALSE,
        FALSE, 0);
    gtk_box_pack_start(GTK_BOX(button_box_ok_cancel), botao_confirmar, FALSE,
        FALSE, 0);

    gtk_container_add(GTK_CONTAINER(intro_window), vbox_repositorios);
    gtk_box_pack_start(GTK_BOX(vbox_repositorios), banner_intro2, FALSE, FALSE,
        0);
    gtk_box_pack_start(GTK_BOX(vbox_repositorios), label_pacote_a_instalar,
        FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_repositorios), label_repositorios, FALSE,
        FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_repositorios), janela_repositorios, TRUE,
        TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_repositorios), button_box_ok_cancel, FALSE,
        FALSE, 0);

    GIOStatus status_rpm_repositorios = G_IO_STATUS_NORMAL;
    GIOChannel *channel_repositorios = NULL;
    if (g_str_has_suffix(pacote_a_instalar,".cm")){
    	channel_repositorios = g_io_channel_new_file(pacote_a_instalar, "r", NULL);
    } else {
	pacote = g_string_new("");
	pacote = pacote_a_instalar; 
	//pacote = g_strconcat(pacote, pacote_a_instalar, NULL);
    	channel_repositorios = g_io_channel_new_file("/etc/apt/sources.list", "r", NULL);
    } 

    if (!channel_repositorios)
    {
        enviaMensagemErro("O pacote escolhido não existe.");
        return 2;
    }

    gchar *linha_rpm_repositorios = NULL;

    int i = 0;

    while (status_rpm_repositorios == G_IO_STATUS_NORMAL)
    {
        status_rpm_repositorios =
            g_io_channel_read_line(channel_repositorios,
            &linha_rpm_repositorios, NULL, NULL, NULL);
        if (linha_rpm_repositorios != NULL)
        {
            /* if (g_strrstr(linha_rpm_repositorios, "rpm")) */
            if (g_strrstr(linha_rpm_repositorios, "deb") && !g_strrstr(linha_rpm_repositorios, "#"))
            {
                gchar *conteudo_old_school;

                g_object_get(buffer_repositorios, "text", &conteudo_old_school,
                    NULL);

                gchar *string_a_colocar =
                    g_strconcat(conteudo_old_school, linha_rpm_repositorios,
                    NULL);

                gtk_text_buffer_set_text(buffer_repositorios, string_a_colocar,
                    -1);

                g_free(string_a_colocar);
                g_free(conteudo_old_school);

                ++i;
            }
            else
            {
                gchar *string_end =
                    linha_rpm_repositorios + strlen(linha_rpm_repositorios) - 1;

                while (isspace(*string_end))
                {
                    *string_end = '\0';
                    --string_end;
                }

                if (g_str_has_suffix(pacote_a_instalar,".cm")){
			pacote = g_strdup(linha_rpm_repositorios);
		}
                gchar *string_da_label =
                    g_strconcat("O pacote " "", pacote, NULL);
                string_da_label =
                    g_strconcat(string_da_label, "" " será instalado.", NULL);

                gtk_label_set_label(GTK_LABEL(label_pacote_a_instalar),
                    string_da_label);
            }
        }
    }

    g_signal_connect(botao_cancelar, "clicked", G_CALLBACK(retError), NULL);

    g_signal_connect(botao_confirmar, "clicked", G_CALLBACK(iniciaProgresso),
        NULL);

    g_signal_connect(intro_window, "delete-event", gtk_main_quit, NULL);

    gtk_widget_show_all(intro_window);

    gtk_main();

    return 0;
}
